package model

import (
	"context"
	"gorm.io/gorm"
	"yx-dataset-server/app/errors"
	"yx-dataset-server/app/model/entity"
	"yx-dataset-server/app/schema"
)

// NewMenu 创建Menu存储实例
func NewMenu(db *gorm.DB) *Menu {
	return &Menu{db}
}

// Menu Menu存储
type Menu struct {
	db *gorm.DB
}

func (a *Menu) getQueryOption(opts ...schema.MenuQueryOptions) schema.MenuQueryOptions {
	var opt schema.MenuQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	return opt
}

// Query 查询数据
func (a *Menu) Query(ctx context.Context, params schema.MenuQueryParam, opts ...schema.MenuQueryOptions) (*schema.MenuQueryResult, error) {
	db := entity.GetMenuDB(ctx, a.db)
	if v := params.LikeName; v != "" {
		db = db.Where("name LIKE ?", "%"+v+"%")
	}
	if v := params.RecordIDs; len(v) > 0 {
		db = db.Where("record_id in (?)", v)
	}
	db = db.Order("sequence ASC")

	opt := a.getQueryOption(opts...)
	var list entity.Menus
	pr, err := WrapPageQuery(ctx, db, opt.PageParam, &list)
	if err != nil {
		return nil, errors.ErrDBServerInternalError
	}
	qr := &schema.MenuQueryResult{
		PageResult: pr,
		Data:       list.ToSchemaMenus(),
	}

	return qr, nil
}

// Get 查询指定数据
func (a *Menu) Get(ctx context.Context, recordID string, opts ...schema.MenuQueryOptions) (*schema.Menu, error) {
	db := entity.GetMenuDB(ctx, a.db).Where("record_id=?", recordID)
	var item entity.Menu
	ok, err := FindOne(ctx, db, &item)
	if err != nil {
		return nil, errors.ErrDBServerInternalError
	} else if !ok {
		return nil, nil
	}

	return item.ToSchemaMenu(), nil
}

// Create 创建数据
func (a *Menu) Create(ctx context.Context, item schema.Menu) error {
	Menu := entity.SchemaMenu(item).ToMenu()
	result := entity.GetMenuDB(ctx, a.db).Create(Menu)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// Update 更新数据
func (a *Menu) Update(ctx context.Context, recordID string, item schema.Menu) error {
	Menu := entity.SchemaMenu(item).ToMenu()
	result := entity.GetMenuDB(ctx, a.db).Where("record_id=?", recordID).Omit("record_id", "creator").Updates(Menu)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// Delete 删除数据
func (a *Menu) Delete(ctx context.Context, recordID string) error {
	result := entity.GetMenuDB(ctx, a.db).Where("record_id=? or parent_id = ?", recordID, recordID).Delete(entity.Menu{})
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// UpdateStatus 更新状态
func (a *Menu) UpdateStatus(ctx context.Context, recordID string, status int) error {
	result := entity.GetMenuDB(ctx, a.db).Where("record_id=?", recordID).Update("status", status)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// UpdateSequence 更新排序
func (a *Menu) UpdateSequence(ctx context.Context, recordID string, sequence int) error {
	// 严格按照你的 UpdateStatus 格式编写
	result := entity.GetMenuDB(ctx, a.db).Where("record_id=?", recordID).Update("sequence", sequence)
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// UpdateBatchSequenceMinus 批量排序-1
func (a *Menu) UpdateBatchSequenceMinus(ctx context.Context, oldSequence, sequence int) error {
	result := entity.GetMenuDB(ctx, a.db).Where("sequence <= ? and sequence > ?", sequence, oldSequence).Update("sequence", gorm.Expr("sequence - 1"))
	if err := result.Error; err != nil {
		return errors.ErrDBServerInternalError
	}
	return nil
}

// GetMaxSequence 查询菜单 sequence 排序字段的最大值
func (a *Menu) GetMaxSequence(ctx context.Context) (int, error) {
	var maxSeq int
	result := entity.GetMenuDB(ctx, a.db).Select("MAX(sequence)").Scan(&maxSeq)
	if err := result.Error; err != nil {
		return 0, errors.ErrDBServerInternalError
	}
	return maxSeq, nil
}
